import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URI;
import java.util.ArrayList;

import org.apache.hadoop.filecache.DistributedCache;
import org.apache.hadoop.fs.FileStatus;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Mapper;

/**
 * Map class
 * @author SergeiL
 */
public class ClaraCostMapper extends Mapper<LongWritable, Text, Text, DoubleWritable>
{
	private ArrayList<ArrayList<Point>> mMedoids;

	@Override
	protected void map(LongWritable key, Text value, Context context) throws IOException, InterruptedException
	{
		Point p = parseData(value);
		double dist;
		double min = Double.MAX_VALUE;
		int idx = -1;
		int resultIdx = 0;
		
		for (ArrayList<Point> medoids : mMedoids)
		{
			min = Long.MAX_VALUE;
			idx++;
			
			for (Point point : medoids)
			{
				dist = calcDistance(p.getX(), point.getX(), p.getY(), point.getY());
				if(dist < min) 
				{
					min = dist;
					resultIdx = idx;
				}
			}
			
			context.write(new Text(String.valueOf(resultIdx)), new DoubleWritable(min));
		}
	}
	
	public Point parseData(Text value) {
		String[] valueArr;		
		valueArr = value.toString().split(";");		
		return new Point(Integer.parseInt(valueArr[0]), Integer.parseInt(valueArr[1]));
	}
	
	private double calcDistance(long x1, long x2, long y1, long y2) {
		// Double.parseDouble(new
		// DecimalFormat("#.##").format(unformattedNumber));
		return Math.sqrt(Math.pow((x2 - x1), 2) + Math.pow((y2 - y1), 2));
	}

	@Override
	protected void setup(Context context) throws IOException, InterruptedException
	{
		try
		{
			// DistributedCache.getCacheFiles()
			// Path my_path=new Path(uris[0].getPath());
			// http://mail-archives.apache.org/mod_mbox/hadoop-common-user/201007.mbox/%3C651981.45757.qm@web46311.mail.sp1.yahoo.com%3E
			// Path[] cacheFiles =
			// DistributedCache.getLocalCacheFiles(context.getConfiguration());
			URI[] cacheFiles = DistributedCache.getCacheFiles(context.getConfiguration());
			mMedoids = new ArrayList<ArrayList<Point>>();
			String dataPath = context.getConfiguration().get("GROUPED_DATA_PATH");

			if (cacheFiles != null && cacheFiles.length > 0)
			{
				for (URI cachePath : cacheFiles)
				{
					//System.out.println("cachePath.getPath(): " + cachePath.getPath());
					if (cachePath.getPath().equals(dataPath))
					{
						//System.out.println("Loading dataPath: " + dataPath);
						loadMedoidData(cachePath.getPath(), context);						
						break;
					}
				}
			}
		}
		catch (IOException ioe)
		{
			System.err.println("IOException reading from distributed cache");
			System.err.println(ioe.toString());
		}

		super.setup(context);
	}
	
	private void loadMedoidData(String loc, Context context) throws IOException
	{
		FileSystem fs = FileSystem.get(context.getConfiguration());
		FileStatus[] status = fs.listStatus(new Path(loc));
		ArrayList<Point> interRes;
		String line;
		String[] points;
		String[] pp;	
		BufferedReader br = null;
		
		for (int i=0;i<status.length;i++)
		{
			if(status[i].getPath().getName().indexOf("part-r-") == -1 || status[i].getPath().getName().indexOf(".crc") != -1) continue;
            br = new BufferedReader(new InputStreamReader(fs.open(status[i].getPath())));
            
            while ((line = br.readLine()) != null)
			{
				interRes = new ArrayList<Point>();
				points = line.split(":")[1].split(",");
				
				for (String p : points)
				{
					pp = p.split(";");
					interRes.add(new Point(Integer.parseInt(pp[0]), Integer.parseInt(pp[1])));
				}
				
				mMedoids.add(interRes);
			}
		}
		
		if(br != null) br.close();
	}
}