import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.filecache.DistributedCache;
import org.apache.hadoop.fs.FileStatus;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.input.TextInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.mapreduce.lib.output.TextOutputFormat;


public class Main {
	
	private static Configuration mConfig;
	private ArrayList<ArrayList<Point>> mMedoids;

	//TODO: Get those values from main function args
	private static String TEST_DATA_PATH_IN;
	private static String TEST_DATA_PATH_OUT;
	private static String MEDOID_DATA_PATH_OUT;
	private static String GROUPED_DATA_PATH;
	private static int NUM_OF_REDUCE_TASKS;
	private static int NUM_OF_EXPERIMENTS;
	private static int NUM_OF_MEDOIDS;
	private static int DATA_SAMPLE_SIZE;
	
	/**
	 * @param args
	 * @throws Exception 
	 */
	public static void main(String[] args) throws Exception {
		double start = System.currentTimeMillis();
		DateFormat dateFormat = new SimpleDateFormat("yyyy_MM_dd_HH_mm_ss");
		Date date = new Date();
		TEST_DATA_PATH_OUT = "Sergei_Laada/HadoopClara/" + dateFormat.format(date);
		GROUPED_DATA_PATH = TEST_DATA_PATH_OUT + "/GroupedData/";
		MEDOID_DATA_PATH_OUT = TEST_DATA_PATH_OUT + "/MedoidData/";
		
		DATA_SAMPLE_SIZE = 100;//40 + 6 * 2;
		NUM_OF_MEDOIDS = 5;
		NUM_OF_EXPERIMENTS = 3;
		NUM_OF_REDUCE_TASKS = 1;
		TEST_DATA_PATH_IN = "Sergei_Laada/HadoopClara/testData1m.csv";
		
		if(args != null && args.length > 0 && args[0] != null) DATA_SAMPLE_SIZE = Integer.parseInt(args[0]);
		if(args != null && args.length > 1 && args[1] != null) NUM_OF_MEDOIDS = Integer.parseInt(args[1]);
		if(args != null && args.length > 2 && args[2] != null) NUM_OF_EXPERIMENTS = Integer.parseInt(args[2]);
		if(args != null && args.length > 3 && args[3] != null) TEST_DATA_PATH_IN = args[3];
		if(args != null && args.length > 4 && args[4] != null) NUM_OF_REDUCE_TASKS = Integer.parseInt(args[4]);
		
		Main prog = new Main();
		mConfig = new Configuration();
		mConfig.setInt("NUM_OF_EXPERIMENTS", NUM_OF_EXPERIMENTS);
		mConfig.setInt("NUM_OF_MEDOIDS", NUM_OF_MEDOIDS);
		mConfig.setInt("DATA_SAMPLE_SIZE", DATA_SAMPLE_SIZE);
		mConfig.set("GROUPED_DATA_PATH", GROUPED_DATA_PATH.substring(0, GROUPED_DATA_PATH.length() - 1));
		
		prog.runGrouperJob(TEST_DATA_PATH_IN, GROUPED_DATA_PATH);
		prog.runCostJob(TEST_DATA_PATH_IN, MEDOID_DATA_PATH_OUT);
		prog.loadMedoidData(GROUPED_DATA_PATH, mConfig);
		prog.loadData(mConfig);

		System.out.println("-----RESULT TIME: " + ((System.currentTimeMillis() - start) / 1000));
	}
	
	private void loadMedoidData(String loc, Configuration conf) throws IOException
	{
		FileSystem fs = FileSystem.get(conf);
		FileStatus[] status = fs.listStatus(new Path(loc));
		ArrayList<Point> interRes;
		String line;
		String[] points;
		String[] pp;	
		BufferedReader br = null;
		mMedoids = new ArrayList<ArrayList<Point>>();
		
		for (int i=0;i<status.length;i++)
		{
			if(status[i].getPath().getName().indexOf("part-r-") == -1 || status[i].getPath().getName().indexOf(".crc") != -1) continue;
            br = new BufferedReader(new InputStreamReader(fs.open(status[i].getPath())));
            
            while ((line = br.readLine()) != null)
			{
				interRes = new ArrayList<Point>();
				points = line.split(":")[1].split(",");
				
				for (String p : points)
				{
					pp = p.split(";");
					interRes.add(new Point(Integer.parseInt(pp[0]), Integer.parseInt(pp[1])));
				}
				
				mMedoids.add(interRes);
			}
		}
		
		if(br != null) br.close();
	}
	
	private ArrayList<Point> loadData(Configuration conf) throws IOException
	{
		BufferedReader reader = null;
		String line;
		double min = Double.MAX_VALUE;
		ArrayList<Point> medoid = null;
		double dist = 0;
		FileSystem fs = FileSystem.get(mConfig);
		FileStatus[] status = fs.listStatus(new Path(MEDOID_DATA_PATH_OUT));
		
		for (int i=0;i<status.length;i++)
		{
			if(status[i].getPath().getName().indexOf("part-r-") == -1 || status[i].getPath().getName().indexOf(".crc") != -1) continue;
			reader = new BufferedReader(new InputStreamReader(fs.open(status[i].getPath())));
			
			while ((line = reader.readLine()) != null)
			{
				dist = Double.parseDouble(line.split("	")[1]);
				System.out.println("POSSIBLE MEDOID: " + line);
				if(dist < min) 
				{
					min = dist;
					medoid = mMedoids.get(Integer.parseInt(line.split("	")[0]));
				}
			}
		}
		
		if(reader != null) reader.close();
		System.out.println("selected medoids: " + medoid.toString());
		return medoid;
	}
	
	/**
	 * Run function
	 * @param pathin - path in
	 * @param pathout - path out
	 * @return int
	 * @throws Exception
	 */
	public int runGrouperJob(String pathin, String pathout) throws Exception
	{
		Job job = new Job(mConfig, "Clara Grouper job");
		job.setJarByClass(Main.class);

		job.setMapperClass(ClaraGroupMapper.class);
		job.setReducerClass(ClaraGroupReducer.class);

		job.setInputFormatClass(TextInputFormat.class);
		job.setOutputFormatClass(TextOutputFormat.class);

		job.setMapOutputKeyClass(LongWritable.class);
		job.setMapOutputValueClass(Text.class);

		job.setOutputKeyClass(LongWritable.class);
		job.setOutputValueClass(Text.class);

		job.setNumReduceTasks(NUM_OF_REDUCE_TASKS);

		FileInputFormat.addInputPath(job, new Path(pathin));
		FileOutputFormat.setOutputPath(job, new Path(pathout));

		boolean success = job.waitForCompletion(true);
		return success ? 0 : -1;
	}
	
	public int runCostJob(String pathin, String pathout) throws Exception
	{
		Job job = new Job(mConfig, "Clara Cost job");
		cacheFile(job, GROUPED_DATA_PATH.substring(0, GROUPED_DATA_PATH.length() - 1));
		job.setJarByClass(Main.class);

		job.setMapperClass(ClaraCostMapper.class);
		job.setReducerClass(ClaraCostReducer.class);

		job.setInputFormatClass(TextInputFormat.class);
		job.setOutputFormatClass(TextOutputFormat.class);

		job.setMapOutputKeyClass(Text.class);
		job.setMapOutputValueClass(DoubleWritable.class);

		job.setOutputKeyClass(Text.class);
		job.setOutputValueClass(DoubleWritable.class);

		job.setNumReduceTasks(NUM_OF_REDUCE_TASKS);

		FileInputFormat.addInputPath(job, new Path(pathin));
		FileOutputFormat.setOutputPath(job, new Path(pathout));

		boolean success = job.waitForCompletion(true);
		return success ? 0 : -1;
	}
	
	/**
	 * Cache train file
	 * @param job - job instance
	 * @throws IOException
	 */
	private static void cacheFile(Job job, String path) throws IOException
	{
		//FileSystem fs = FileSystem.get(job.getConfiguration());
		//System.out.println("path: "+path);
		Path hdfsPath = new Path(path);
		// upload the file to hdfs. Overwrite any existing copy.
		// http://stackoverflow.com/questions/15434709/checksum-exception-when-reading-from-or-copying-to-hdfs-in-apache-hadoop
		//fs.copyFromLocalFile(false, true, new Path(TRAIN_DATA_PATH), hdfsPath);
		DistributedCache.addCacheFile(hdfsPath.toUri(), job.getConfiguration());
	}
}