import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataInputStream;
import org.apache.hadoop.fs.FileStatus;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.Text;

/**
 * Helper class
 * @author SergeiL
 */
public class Helper
{
	/**
	 * Method to read numbers (factors) from files in a folder in HDFS
	 */
	public static ArrayList<String> readResult(String pathStr, Configuration config) throws IOException
	{
		ArrayList<String> result = new ArrayList<String>();

		FileSystem fs = FileSystem.get(config);
		Path path = new Path(pathStr);

		if (fs.getFileStatus(path).isDir())
		{
			FileStatus[] listFiles = fs.listStatus(path);

			for (int i = 0; i < listFiles.length; i++)
			{
				try
				{
					if (listFiles[i].isDir()) continue;
					// Get the object of DataInputStream
					FSDataInputStream in = fs.open(listFiles[i].getPath());
					BufferedReader br = new BufferedReader(new InputStreamReader(in));
					String strLine;
					// Read File Line By Line
					while ((strLine = br.readLine()) != null) result.add(strLine);
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}
			}
		}

		return result;
	}

	/**
	 * Load train data file
	 * @param cachePath - cache path
	 * @return ArrayList<DataField>
	 * @throws IOException
	 */
	public static ArrayList<DataField> loadTrainDataFile(String cachePath) throws IOException
	{
		// note use of regular java.io methods here - this is a local file now
		BufferedReader reader = new BufferedReader(new FileReader(cachePath));
		ArrayList<DataField> result;
		String sex;
		double height;
		double weight;
		double footSize;

		try
		{
			String line;
			DataField field;
			String[] lineArr;
			result = new ArrayList<DataField>();

			while ((line = reader.readLine()) != null)
			{
				lineArr = line.split(Main.LINE_SEPARATOR);
				sex = lineArr[0];
				height = Double.parseDouble(lineArr[1]);
				weight = Double.parseDouble(lineArr[2]);
				footSize = Double.parseDouble(lineArr[3]);
				field = new DataField(sex, height, weight, footSize);
				result.add(field);
			}
		}
		finally
		{
			reader.close();
		}

		return result;
	}

	/**
	 * Calculate mean
	 * @param data - data array
	 * @return double
	 */
	public static double calcMean(double[] data)
	{
		double result = 0;
		for (int i = 0; i < data.length; i++) result += data[i];
		result = result / data.length;
		return result;
	}

	/**
	 * Calculate variance
	 * @param data - data array
	 * @param mean - mean
	 * @return double
	 */
	public static double calcVariance(double[] data, double mean)
	{
		double result = 0;
		for (int i = 0; i < data.length; i++) result += Math.pow(data[i] - mean, 2);
		result /= (data.length - 1);
		return result;
	}

	/**
	 * Text to data
	 * @param key - key string
	 * @param data - data array
	 * @return ArrayList<DataField>
	 */
	public static ArrayList<DataField> textToData(String key, Iterable<Text> data)
	{
		ArrayList<DataField> result = new ArrayList<DataField>();
		DataField field;
		double height;
		double weight;
		double footSize;
		String[] split;

		for (Text value : data)
		{
			split = value.toString().split(Main.LINE_SEPARATOR);
			height = Double.parseDouble(split[0]);
			weight = Double.parseDouble(split[1]);
			footSize = Double.parseDouble(split[2]);
			field = new DataField(key, height, weight, footSize);
			result.add(field);
		}

		return result;
	}
}