import java.util.ArrayList;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.input.TextInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.mapreduce.lib.output.TextOutputFormat;

/**
 * Main class
 * @author SergeiL
 */
public class Main
{
	private static Configuration mConfig;

	//TODO: get them from main function args
	private static final String TEST_DATA_PATH_IN = "trainData.csv";
	private static final String TEST_DATA_PATH_OUT = "output";
	private static final int NUM_OF_REDUCE_TASKS = 5;
	public static final String LINE_SEPARATOR = ";";

	/**
	 * Main function
	 * @param args - arguments
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception
	{
		// trainData.csv testData.csv output 1 5
		Main prog = new Main();

		mConfig = new Configuration();
		FileSystem fs = FileSystem.get(mConfig);

		// Delete output folder if it already exists
		fs.delete(new Path(TEST_DATA_PATH_OUT), true);

		prog.run(TEST_DATA_PATH_IN, TEST_DATA_PATH_OUT);
		ArrayList<String> results = Helper.readResult(TEST_DATA_PATH_OUT, mConfig);

		for (String result : results) System.out.println(result);
	}

	/**
	 * Run job
	 * @param pathin - path in
	 * @param pathout - path out
	 * @return int
	 * @throws Exception
	 */
	public int run(String pathin, String pathout) throws Exception
	{
		Job job = new Job(mConfig, "PnBm train job");
		job.setJarByClass(Main.class);

		job.setMapperClass(PnBmTrainMapper.class);
		job.setReducerClass(PnBmTrainReducer.class);

		job.setInputFormatClass(TextInputFormat.class);
		job.setOutputFormatClass(TextOutputFormat.class);

		job.setMapOutputKeyClass(Text.class);
		job.setMapOutputValueClass(Text.class);

		job.setOutputKeyClass(Text.class);
		job.setOutputValueClass(Text.class);

		job.setNumReduceTasks(NUM_OF_REDUCE_TASKS);

		FileInputFormat.addInputPath(job, new Path(pathin));
		FileOutputFormat.setOutputPath(job, new Path(pathout));

		boolean success = job.waitForCompletion(true);
		return success ? 0 : -1;
	}
}