import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.Hashtable;
import java.util.List;
import org.apache.spark.api.java.*;
import org.apache.spark.api.java.function.*;

/**
 * Class for parallel k-nearest neighbor algorithm
 * @author SergeiL
 */
public class Main {
	// Train data
	private static JavaRDD<DataField> mTrainData;
	
	private static int NUM_OF_NEIGH;

	/**
	 * Main function
	 * @param args
	 * args[0] - train data path 
	 * args[1] - test data path 
	 * args[2] - number of neighbors
	 */
	public static void main(String[] args) {
		double start = System.currentTimeMillis();
		NUM_OF_NEIGH = 5;
		String testDataPath = "testData.csv";
		String trainDataPath = "trainData.csv";
		int procNum = 1;
		
		if(args != null && args.length > 0 && args[0] != null) NUM_OF_NEIGH = Integer.parseInt(args[0]);
		if(args != null && args.length > 1 && args[1] != null) testDataPath = args[1];
		if(args != null && args.length > 2 && args[2] != null) procNum = Integer.parseInt(args[2]);
		if(args != null && args.length > 3 && args[3] != null) trainDataPath = args[3];

		DateFormat dateFormat = new SimpleDateFormat("yyyy_MM_dd_HH_mm_ss");
		Date date = new Date();
		
		JavaSparkContext sc = new JavaSparkContext("spark://laada-spark-master:7077", "Spark PkNN App", "/root/spark-0.8.0-incubating", "SparkPkNN.jar");
		mTrainData = sc.textFile("hdfs://laada-spark-master:54310/user/hduser/SparkPkNN/" + trainDataPath).map(new ParseData());
		JavaRDD<DataField> testData = sc.textFile("hdfs://laada-spark-master:54310/user/hduser/SparkPkNN/" + testDataPath, procNum).map(new ParseData());
		JavaRDD<String> mapData = testData.map(new MapElement(mTrainData.collect(), NUM_OF_NEIGH));
		mapData.saveAsTextFile("hdfs://laada-spark-master:54310/user/hduser/SparkPkNN/result/" + dateFormat.format(date));
		System.out.println("-----RESULT TIME: " + ((System.currentTimeMillis() - start) / 1000));
	}
}

/**
 * Map class
 * @author SergeiL
 */
@SuppressWarnings("serial")
class MapElement extends Function<DataField, String> {
	// Train data
	private List<DataField> mTrainData;
	// Number of neighbors
	private int mNumOfNeighbors;

	/**
	 * Constructor
	 * @param values - string values
	 */
	MapElement(List<DataField> values, int neigh) {
		mTrainData = values;
		mNumOfNeighbors = neigh;
	}

	/**
	 * Process data
	 */
	public String call(DataField value) {
		double distance;
		DataField field;
		ArrayList<DataField> fields = new ArrayList<DataField>();
		long maxGroup = Long.MIN_VALUE;
		int maxGroupCount = Integer.MIN_VALUE;
		Hashtable<Long, Integer> groups = new Hashtable<Long, Integer>();
		DataField f;

		for (DataField ft : mTrainData) {
			distance = calcDistance(value.getX(), ft.getX(), value.getY(), ft.getY());
			field = new DataField(value.getId(), distance, ft.getGroup());
			fields.add(field);
		}
		
		Collections.sort(fields);

		for (int i = 0; i < mNumOfNeighbors; i++) {
			f = fields.get(i);
			if (groups.containsKey(f.getGroup()))
				groups.put(f.getGroup(), groups.get(f.getGroup()) + 1);
			else
				groups.put(f.getGroup(), 1);
			if (groups.get(f.getGroup()) > maxGroupCount) {
				maxGroupCount = groups.get(f.getGroup());
				maxGroup = f.getGroup();
			}
		}

		return (value.getId() + ";" + String.valueOf(maxGroup));
	}

	private double calcDistance(long x1, long x2, long y1, long y2) {
		// Double.parseDouble(new
		// DecimalFormat("#.##").format(unformattedNumber));
		return Math.sqrt(Math.pow((x2 - x1), 2) + Math.pow((y2 - y1), 2));
	}
}

/**
 * Parse data class
 * @author SergeiL
 */
@SuppressWarnings("serial")
class ParseData extends Function<String, DataField> {
	
	/**
	 * Process data
	 */
	public DataField call(String value) {
		String[] valueArr = value.split(";");
		if(valueArr.length == 3) return new DataField(Long.parseLong(valueArr[0]), Long.parseLong(valueArr[1]), Long.parseLong(valueArr[2]));
		return new DataField(Long.parseLong(valueArr[0]), Long.parseLong(valueArr[1]), Long.parseLong(valueArr[2]), Long.parseLong(valueArr[3]));
	}
}